package dk.sundhedsdatastyrelsen.cbs.datacache.invalidation;

import dk.sundhedsdatastyrelsen.cbs.dao.LabResultRecord;
import dk.sundhedsdatastyrelsen.cbs.dao.VaccinationRecord;
import dk.sundhedsdatastyrelsen.cbs.datacache.Writer;
import dk.sundhedsdatastyrelsen.cbs.datacache.invalidation.impl.CacheInvalidatingNasNotificationProcessor;
import dk.sundhedsdatastyrelsen.cbs.datacache.invalidation.impl.InMemoryNasSubscriptionRepository;
import dk.sundhedsdatastyrelsen.cbs.datacache.invalidation.impl.LabResultRecordInvalidator;
import dk.sundhedsdatastyrelsen.cbs.datacache.invalidation.impl.VaccinationRecordInvalidator;
import dk.sundhedsdatastyrelsen.cbs.dgws.DGWSRequestFactory;
import dk.sundhedsdatastyrelsen.cbs.dgws.STSConfig;
import dk.sundhedsdatastyrelsen.cbs.dgws.http.HttpClientService;
import dk.sundhedsdatastyrelsen.cbs.dgws.http.HttpClientServiceImpl;
import dk.sundhedsdatastyrelsen.cbs.dgws.http.HttpExecutor;
import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResult;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.Vaccination;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.DGWSHeaderProvider;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.DefaultDGWSHeaderProvider;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.NasClient;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.NasClientImpl;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.NasConsumer;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.NasConsumerProperties;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.NasNotificationProcessor;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.NasSubscriptionRepository;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.PullPointFactory;
import dk.vaccinationsregister.schemas._2021._04._01.Covid19VaccinationCardType;
import org.oasis_open.docs.wsn.bw_2.CreatePullPoint;
import org.oasis_open.docs.wsn.bw_2.NotificationProducer;
import org.oasis_open.docs.wsn.bw_2.PullPointFactoryService;
import org.oasis_open.docs.wsn.bw_2.SubscriptionManagerService;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.xml.bind.JAXBException;
import javax.xml.ws.BindingProvider;

@Configuration
public class DataCacheInvalidationConfiguration {

    @Bean
    @Qualifier("vaccinationNasConsumer")
    NasConsumer vaccinationNasConsumer(
        NasConsumerProperties vaccinationNasConsumerProperties,
        NasClient nasClient,
        NasSubscriptionRepository repository,
        NasNotificationProcessor vaccinationNotificationProcessor
    ) {
        return new NasConsumer(vaccinationNasConsumerProperties, nasClient, repository, vaccinationNotificationProcessor);
    }

    @Bean
    NasConsumerProperties vaccinationNasConsumerProperties(
        @Value("${vacc.topic}") String topic,
        @Value("${vacc.chunk.size}") long chunkSize,
        @Value("${vacc.chunk.limit}") int chunkMaxLimit
    ) {
        return new NasConsumerProperties(topic, chunkSize, chunkMaxLimit);
    }

    @Bean
    NasNotificationProcessor vaccinationNotificationProcessor(VaccinationRecordInvalidator invalidator, @Value("${vacc.offset.minutes:75}") long offset) {
        return new CacheInvalidatingNasNotificationProcessor(invalidator, offset);
    }

    @Bean
    VaccinationRecordInvalidator vaccinationRecordInvalidator(Writer<VaccinationRecord<Vaccination, Covid19VaccinationCardType>> writer) {
        return new VaccinationRecordInvalidator(writer);
    }

    @Bean
    @Qualifier("labResultNasConsumer")
    NasConsumer labResultNasConsumer(
        NasConsumerProperties labResultNasConsumerProperties,
        NasClient nasClient,
        NasSubscriptionRepository repository,
        NasNotificationProcessor labResultNotificationProcessor
    ) {
        return new NasConsumer(labResultNasConsumerProperties, nasClient, repository, labResultNotificationProcessor);
    }

    @Bean
    NasConsumerProperties labResultNasConsumerProperties(
        @Value("${lab.topic}") String topic,
        @Value("${lab.chunk.size}") long chunkSize,
        @Value("${lab.chunk.limit}") int chunkMaxLimit
    ) {
        return new NasConsumerProperties(topic, chunkSize, chunkMaxLimit);
    }

    @Bean
    NasNotificationProcessor labResultNotificationProcessor(LabResultRecordInvalidator invalidator, @Value("${lab.offset.minutes:75}") long offset) {
        return new CacheInvalidatingNasNotificationProcessor(invalidator, offset);
    }

    @Bean
    LabResultRecordInvalidator labResultRecordInvalidator(Writer<LabResultRecord<LabResult, Covid19VaccinationCardType>> writer) {
        return new LabResultRecordInvalidator(writer);
    }

    @Bean
    NasSubscriptionRepository repository(NasClient nasClient) {
        return new InMemoryNasSubscriptionRepository(nasClient);
    }

    @Bean
    NasClient nasClient(DGWSHeaderProvider dgwsHeaderProvider, NotificationProducer notificationProducer, CreatePullPoint createPullPoint) {
        return new NasClientImpl(dgwsHeaderProvider, notificationProducer, createPullPoint, new PullPointFactory());
    }

    @Bean
    DGWSHeaderProvider dgwsHeaderProvider(DGWSRequestFactory requestFactory) throws JAXBException {
        return new DefaultDGWSHeaderProvider(requestFactory);
    }

    @Bean
    DGWSRequestFactory dgwsRequestFactory(STSConfig stsConfig, HttpClientService httpClientService) {
        return new DGWSRequestFactory(stsConfig, httpClientService);
    }

    @Bean
    HttpClientService httpClientService() {
        return new HttpClientServiceImpl(new HttpExecutor());
    }

    @Bean
    NotificationProducer notificationProducer(@Value("${nas.subscriptionmanager.endpoint}") String endpoint) {
        NotificationProducer notificationProducer = new SubscriptionManagerService().getSoap();

        BindingProvider bindingProvider = (BindingProvider) notificationProducer;

        bindingProvider.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, endpoint);

        return notificationProducer;
    }

    @Bean
    CreatePullPoint createPullPoint(@Value("${nas.pullpointfactory.endpoint}") String endpoint) {
        CreatePullPoint createPullPoint = new PullPointFactoryService().getSoap();

        BindingProvider bindingProvider = (BindingProvider) createPullPoint;

        bindingProvider.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, endpoint);

        return createPullPoint;
    }

    @Bean
    public STSConfig stsConfig(
        @Value("${nas.idcard.sts.endpoint}") String stsEndpoint,
        @Value("${nas.idcard.name}") String organizationName,
        @Value("${nas.idcard.cvr}") String organizationCode,
        @Value("${nas.idcard.keystore.path}") String keystorePath,
        @Value("${nas.idcard.keystore.password}") String keystorePassword,
        @Value("${nas.idcard.systemName}") String systemName) {
        return new STSConfig(stsEndpoint, organizationName, organizationCode, keystorePath, keystorePassword, systemName);
    }

}
