package dk.sundhedsdatastyrelsen.cbs.datacache.invalidation.impl;

import dk.nsi.advis.v10.NotifyContent;
import dk.sundhedsdatastyrelsen.cbs.datacache.invalidation.CacheInvalidationRecord;
import dk.sundhedsdatastyrelsen.cbs.datacache.invalidation.RecordInvalidator;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.NasNotificationProcessor;
import org.oasis_open.docs.wsn.b_2.NotificationMessageHolderType;

import java.time.LocalDateTime;
import java.util.List;

public class CacheInvalidatingNasNotificationProcessor implements NasNotificationProcessor {

    private final RecordInvalidator recordInvalidator;
    private final long invalidationTimeOffsetInMinutes;

    public CacheInvalidatingNasNotificationProcessor(RecordInvalidator recordInvalidator, long invalidationTimeOffsetInMinutes) {
        this.recordInvalidator = recordInvalidator;
        this.invalidationTimeOffsetInMinutes = invalidationTimeOffsetInMinutes;
    }

    @Override
    public void process(List<NotificationMessageHolderType> messages) {
        for (NotificationMessageHolderType message : messages) {
            CacheInvalidationRecord record = toRecord(message);
            if (record != null) {
                invalidateRecord(record);
            }
        }
    }

    private void invalidateRecord(CacheInvalidationRecord record) {
        boolean invalidated = recordInvalidator.invalidate(record);

        if (!invalidated) {
            handleRecordNotInvalidated(record);
        }
    }

    private CacheInvalidationRecord toRecord(NotificationMessageHolderType message) {
        Object content = message.getMessage().getAny();

        if (content instanceof NotifyContent) {
            NotifyContent notifyContent = (NotifyContent) content;
            return new CacheInvalidationRecord(notifyContent.getId(), getTraceId(message), getInvalidationTime());
        }

        return null;
    }

    private LocalDateTime getInvalidationTime() {
        return LocalDateTime.now().plusMinutes(invalidationTimeOffsetInMinutes);
    }

    private String getTraceId(NotificationMessageHolderType message) {
        return message.getTopic().getContent().get(0).toString();
    }

    private void handleRecordNotInvalidated(CacheInvalidationRecord record) {
        throw new DataCacheInvalidationException(String.format("Could not invalidate record. Trace = %s", record.getTraceId()));
    }


}
