package dk.sundhedsdatastyrelsen.cbs.datacache.invalidation.impl;

import dk.sundhedsdatastyrelsen.cbs.nas.facade.EndpointReferences;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.NasClient;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.NasSubscriptionRepository;

import javax.xml.ws.wsaddressing.W3CEndpointReference;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public class InMemoryNasSubscriptionRepository implements NasSubscriptionRepository {

    private final NasClient nasClient;

    private final Map<String, String> pullPointMap = new HashMap<>();
    private final Map<String, String> messageIdMap = new HashMap<>();


    public InMemoryNasSubscriptionRepository(NasClient nasClient) {
        this.nasClient = nasClient;
    }

    @Override
    public W3CEndpointReference getPullPoint(String topic) {
        if (pullPointMap.containsKey(topic)) {
            String address = pullPointMap.get(topic);
            return EndpointReferences.newEndpointReference(address);
        } else {
            W3CEndpointReference pullPoint = nasClient.createPullPoint();
            nasClient.subscribe(pullPoint, topic);
            pullPointMap.put(topic, EndpointReferences.extractAddress(pullPoint));
            return pullPoint;
        }
    }

    @Override
    public String getNextMessageId(String topic) {
        return messageIdMap.computeIfAbsent(topic, ignored -> UUID.randomUUID().toString());
    }

    @Override
    public void commit(String topic) {
        messageIdMap.remove(topic);
    }
}
