/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.datafetcher.config;

import dk.sundhedsdatastyrelsen.cbs.nas.facade.NasPublisher;
import dk.sundhedsdatastyrelsen.cbs.nas.facade.NasPublisherImpl;
import org.apache.commons.lang3.StringUtils;
import org.oasis_open.docs.wsn.bw_2.NotificationBrokerService;
import org.oasis_open.docs.wsn.bw_2.NotificationConsumer;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.xml.ws.BindingProvider;

@Configuration
public class DataFetcherNasConfig {

    @Bean
    public NasPublisher nasPublisher(@Value("${nas.topic}") String nasTopic,
                                     @Value("${nas.endpoint}") String nasEndpoint,
                                     @Value("${nas.connection-timeout-millis}") String nasConnectionTimeoutMillis,
                                     @Value("${nas.receive-timeout-millis}") String nasReceiveTimeoutMillis) {
        NotificationConsumer notificationConsumer = new NotificationBrokerService().getSoap();
        ((BindingProvider) notificationConsumer).getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, nasEndpoint);
        if (StringUtils.isNotBlank(nasConnectionTimeoutMillis)) {
            //vendor/jdk specific (not sure which one is needed, so setting both)
            ((BindingProvider) notificationConsumer).getRequestContext().put("javax.xml.ws.client.connectionTimeout", nasConnectionTimeoutMillis);
            ((BindingProvider) notificationConsumer).getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", nasConnectionTimeoutMillis);
        }
        if (StringUtils.isNotBlank(nasReceiveTimeoutMillis)) {
            //vendor/jdk specific (not sure which one is needed, so setting both)
            ((BindingProvider) notificationConsumer).getRequestContext().put("javax.xml.ws.client.receiveTimeout", nasReceiveTimeoutMillis);
            ((BindingProvider) notificationConsumer).getRequestContext().put("com.sun.xml.internal.ws.request.timeout", nasReceiveTimeoutMillis);
        }
        return new NasPublisherImpl(notificationConsumer, nasTopic);
    }
}
